from random import randint

import bs4

from scraper import find_term


def make_fake_soup(term):
    lists = ['<ul></ul>' for i in xrange(31)]

    # Subtract one since xrange isn't inclusive
    index = randint(0, 30)

    lists[index] = '<ul><li>%s</li></ul>' % term

    html_list = ['<html><body>']
    html_list.extend(lists)
    html_list.extend(['</body></html>'])

    html = ''.join(html_list)

    soup = bs4.BeautifulSoup(html)

    return soup, index

def test_find_term():
    term = 'Thing'
    soup, index = make_fake_soup(term)

    returned_index = find_term(soup, term)
    assert returned_index == index



import json
import os

import pytest
import bungie

# Get the API response file once, since it won't change.
test_dir = os.path.dirname(os.path.realpath(__file__))
_api_file = open(test_dir + '/fixtures/full_response.json', 'r')
FULL_API_RESPONSE = json.load(_api_file)
API_RESPONSE = FULL_API_RESPONSE['Response']

@pytest.fixture(autouse=True)
def no_request_get(monkeypatch):
    # Set up a mock API response that will be present on all tests,
    # so we don't have to hit the network for testing.
    def mock_fetch_activities(url):
        return API_RESPONSE
    monkeypatch.setattr(bungie, 'fetch_activities', mock_fetch_activities)

def test_process_activities():
    api_response = bungie.fetch_activities('url')

    activities = bungie.process_activities(api_response)

    names = ('nightfall', 'crucible', 'daily', 'heroic')
    assert all(name in activities.keys() for name in names)
    assert all(activities[name] is not None for name in names)

def test_process_nightfall():
    api_response = bungie.fetch_activities('url')

    activities = bungie.process_activities(api_response)

    nf = bungie.process_nightfall(activities)

    keys = ('title', 'description', 'mods', 'name')
    assert all(key in nf.keys() for key in keys)
    assert 'Nightfall' in nf['title']
    assert 'Will of Crota' in nf['name']
    assert len(nf['mods']) == 5
    assert 'Void Burn' in nf['mods']
    assert 'Arc Burn' in nf['mods']
    assert 'Solar Burn' in nf['mods']


def test_process_heroic():
    api_response = bungie.fetch_activities('url')

    activities = bungie.process_activities(api_response)

    heroic = bungie.process_heroic(activities)

    keys = ('title', 'description', 'mods', 'name')
    assert all(key in heroic.keys() for key in keys)
    assert 'Heroic' in heroic['title']
    assert 'Will of Crota' in heroic['name']
    assert len(heroic['mods']) == 2
    assert 'Heroic' in heroic['mods']
    assert 'Solar Burn' in heroic['mods']

def test_process_daily():
    api_response = bungie.fetch_activities('url')

    activities = bungie.process_activities(api_response)

    daily = bungie.process_daily(activities)

    keys = ('title', 'description', 'mods')
    assert all(key in daily.keys() for key in keys)
    assert 'Siege of the Warmind' in daily['title']
    assert 'Siege of the Warmind' in daily['name']
    assert len(daily['mods']) == 2
    assert 'Heroic' in daily['mods']
    assert 'Angry' in daily['mods']

def test_process_crucible():
    api_response = bungie.fetch_activities('url')

    activities = bungie.process_activities(api_response)

    crucible = bungie.process_crucible(activities)

    keys = ('title', 'description', 'name')
    assert all(key in crucible.keys() for key in keys)
    assert "Executor's Challenge" in crucible['title']
    assert "Executor's Challenge" in crucible['name']



from collections import namedtuple


Strike = namedtuple('Strike', ('name', 'terms'))


class UnknownStrikeError(Exception):
    """Signals that the description text could not be matched to a strike."""


DEVILS_LAIR = Strike("The Devil's Lair", ("sepiks",))
WILL_OF_CROTA = Strike("Will of Crota", ("omnigul",))
SUMMONING_PITS = Strike("The Summoning Pits", ("hellmouth", "find the pit", "powerful abominations"))
NEXUS = Strike("The Nexus", ("nexus",))
WINTERS_RUN = Strike("Winter's Run", ("archon", "house of winter", "prison of elders"))
CERBERUS_VAE_III = Strike("Cerberus Vae III", ("valus ta'aurc", "tank"))
DUST_PALACE = Strike("Dust Palace", ("psion flayers", "martian warmind", "cabal"))
UNDYING_MIND = Strike("Undying Mind", ("ancient mind", "black garden"))
SHADOW_THIEF = Strike("The Shadow Thief", ("ketch", "taniks"))
STRIKES = (
    DEVILS_LAIR,
    WILL_OF_CROTA,
    SUMMONING_PITS,
    NEXUS,
    WINTERS_RUN,
    CERBERUS_VAE_III,
    DUST_PALACE,
    UNDYING_MIND,
    SHADOW_THIEF,
)


def get_short_name(description):
    """Searches strike description text for identifying terms.

    :param description: text to search.
    :returns: Short name for the strike.
    :raises UnknownStrikeError: if no matching strike name is found.
    """
    text = description.lower()
    for match in (s for s in STRIKES if all(term in text for term in s.terms)):
        return match.name
    else:
        raise UnknownStrikeError


__all__ = (
    'get_short_name',
    'UnknownStrikeError',
    'DEVILS_LAIR',
    'WILL_OF_CROTA',
    'SUMMONING_PITS',
    'NEXUS',
    'WINTERS_RUN',
    'CERBERUS_VAE_III',
    'DUST_PALACE',
    'UNDYING_MIND',
    'SHADOW_THIEF',
)



from functools import partial
import requests
import bs4


event_url = 'http://destinytracker.com/destiny/events'

class ScrapeError(Exception):
    pass

def get_soup(event_url=event_url):
    response = requests.get(event_url)
    html = response.text

    soup = bs4.BeautifulSoup(response.text)
    return soup


def strike_title(soup):
    title = soup.select('h4.media-heading')[-1].text
    return title


NIGHTFALL = 'nightfall'
WEEKLY_HEROIC = 'weekly heroic'

VENDOR_NAMES = [
    'eris',
    'crucible',
    'vanguard',
]

def find_tables(vendor):
    soup = get_soup("http://db.planetdestiny.com/events")
    headers = soup.select('h3')
    target_index = None
    for index, header in enumerate(headers):
        if vendor in header.text.lower():
            target_index = index
            break

    table = soup.select('table')[target_index]

    return table

def read_bounty_tables(vendor):
    vendor = vendor.lower()
    table = find_tables(vendor)

    # The first row is the heading
    rows = table.select('tr')[1:]

    bounties = {}
    for row in rows:
        cells = row.select('td')
        name = cells[0].select('a')[0].text
        name = name.strip()

        desc = cells[1].text.strip()

        bounties[name] = desc

    return bounties



import json

import requests

from strike import get_short_name

ACTIVITY_URL = 'http://www.bungie.net/Platform/Destiny/Advisors/?definitions=True'

def fetch_activities():
    http_response = requests.get(ACTIVITY_URL)
    json_response = http_response.json()
    api_response = json_response['Response']
    return api_response

def process_activities(api_response):
    """
    Processes a Bungie API response by pulling out activity data from the rest
    of the information. These definitions will be further processed into
    less obscured dictionaries by other functions.

    :param api_response: A dictionary of the original Bungie API response
    :return dictionary: Dictionary mapping the nightfal, heroic, daily, and
                        crucible event information to these names.
    """

    activity_map = {}

    nightfall_hash = str(api_response['data']['nightfallActivityHash'])
    daily_hashes = api_response['data']['dailyChapterHashes']
    heroic_hashes = api_response['data']['heroicStrikeHashes']
    crucible_hash = str(api_response['data']['dailyCrucibleHash'])

    activities = api_response['definitions']['activities']

    activity_map['nightfall'] = activities[nightfall_hash]

    activity_map['crucible'] = activities[crucible_hash]

    dailies_info = []
    for d_hash in daily_hashes:
        dailies_info.append(activities[str(d_hash)])

    activity_map['daily'] = dailies_info

    heroics_info = []
    for h_hash in heroic_hashes:
        heroics_info.append(activities[str(h_hash)])

    activity_map['heroic'] = heroics_info

    return activity_map

def process_nightfall(activity_map):
    """
    :param activity_map: A dictionary of processed event information
    :return dictionary: A dictionary of the Nightfall's information
    """
    nightfall_info = activity_map['nightfall']
    description = nightfall_info['activityDescription']
    skulls = nightfall_info['skulls']
    mods = [s['displayName'] for s in skulls]

    title = nightfall_info['activityName']

    name = get_short_name(description)

    nf_dict = {'title': title,
               'description': description,
               'mods': mods,
               'name': name,
    }

    return nf_dict

def process_heroic(activity_map):
    """
    :param activity_map: A dictionary of processed event information
    :return dictionary: A dictionary of the Heroic's information
    """
    heroic_acts = activity_map['heroic']

    # Here, we'll just use the first instance's information
    # since the only variation should be the levels.
    heroic_info = heroic_acts[0]
    description = heroic_info['activityDescription']
    skulls = heroic_info['skulls']
    mods = [s['displayName'] for s in skulls]

    title = heroic_info['activityName']

    name = get_short_name(description)

    heroic_dict = {'title': title,
                   'description': description,
                   'mods': mods,
                   'name': name,
    }
    return heroic_dict

def process_daily(activity_map):
    """
    :param activity_map: A dictionary of processed event information
    :return dictionary: A dictionary of the Daily's information
    """
    daily_acts = activity_map['daily']

    # Here, we'll just use the first instance's information
    # since the only variation should be the levels.
    daily_info = daily_acts[0]
    description = daily_info['activityDescription']
    skulls = daily_info['skulls']
    mods = [s['displayName'] for s in skulls]

    title = daily_info['activityName']

    daily_dict = {'title': title,
                  'description': description,
                  'mods': mods,
                  'name': title,
    }
    return daily_dict

def process_crucible(activity_map):
    crucible_info = activity_map['crucible']
    description = crucible_info['activityDescription']

    title = crucible_info['activityName']

    crucible_dict = {'title': title,
                     'description': description,
                     'name': title,
    }

    return crucible_dict

def get_activity_details(activity_name):
    funcs = {
        'nightfall': process_nightfall,
        'daily': process_daily,
        'crucible': process_crucible,
        'heroic': process_heroic,
    }

    # TODO: Caching logic could probably go here.
    activity_json = fetch_activities()
    activities = process_activities(activity_json)

    return funcs[activity_name](activities)

if __name__ == '__main__':
    act_json = fetch_activities()
    activities = process_activities(act_json)



import datetime

CACHE = {}

__all__ = ['write', 'read']

def write(key, value, expires):
    """
    Caches a value under a given key that will expire in *at least* a given amount of time

    If an entry already exists for a key, and will expire later than the given
    expire time, the first value is used, and the return value will be `False`.

    On writing a new value, if a current value exists but is expired, the
    current value will be removed, and the new value written.

    Args:
        key - name of the item to cache
        value - the actual value to cache
        expires = a python Datetime (tz-naive) object for when to clear the key
    Returns:
        Boolean - True on successful write, False if a longer-lived value is written
    """
    global CACHE
    value_dict = None
    try:
        value_dict = CACHE[key]
    except KeyError:
        pass

    if value_dict and value_dict.has_key('expires'):
        now = datetime.datetime.now()
        if now > value_dict['expires']:
            CACHE.pop(key)
        # The current expiry is later than the provided one.
        if expires < value_dict['expires']:
            return False

    value_dict = {'value': value, 'expires': expires}
    CACHE[key] = value_dict
    return True


def read(key):
    """
    Returns a value for a given key in the cache.

    If the key is present, and is not expired, the value will be returned

    If the value at the given key is expired, it will be removed from the
    cache and None will be returned.

    If no value exists for a given key, then None will be returned
    """
    global CACHE
    try:
        value_dict = CACHE[key]
    except KeyError:
        return None

    value, expires = value_dict['value'], value_dict['expires']

    now = datetime.datetime.now()
    if now > expires:
        # Value is old, so we shouldn't give it back.
        CACHE.pop(key)
        return None

    return value


def clear():
    """
    Empties the cache entirely
    """
    global CACHE
    CACHE = {}



import pytest
import strike

# In-game descriptions pulled from the internet
DEVILS_LAIR = """The Fallen House of Devils in the Cosmodrome sustain their strength through a single, exalted Servitor known as Sepiks Prime. Find where this keeper of souls lies and destroy him."""
WILL_OF_CROTA = """Omnigul, the mastermind of Crota's armies, nests in the Cosmodrome. Eliminate this horror and purge its spawn."""
SUMMONING_PITS = """Deep within the Hellmouth, the Hive are breeding powerful abominations to unleash upon the Earth. Find the pit where they live and stop any new horrors from rising."""
NEXUS = """The Golden Age legends tell of ancient tunnels below the Academy, where a Vex machine once ate away at the planet. Descend into the old dig sites and find this Nexus before it churns again."""
WINTERS_RUN = """As the Vex march to claim the Ishtar Sink, the Fallen House of Winter are raising a powerful Archon, stolen from the Prison of Elders in the Reef. Find the Archon before they fully restore his soul."""
CERBERUS_VAE_III = """Just beyond their warbase, Valus Ta'aurc, Fleet Commander of the Cabal Siege Dancers, hides in an Imperial Land Tank. Fight through his guard and stop this terror before he destroys all of Freehold."""
DUST_PALACE = """A new Cabal force has unleashed powerful Psion Flayers to seize control of the old Martian Warmind. Face this new threat and secure what remains of the ancient network."""
UNDYING_MIND = "An ancient Mind, feared by the Vex themselves, hides among the channels of the Black Garden. Find this machine and end its existence."
# and then there was this...
PREDATOR = """Dutch (Arnold Schwarzenegger), a soldier of fortune, is hired by the U.S. government to secretly rescue a group of politicians trapped in Guatemala. But when Dutch and his team, which includes weapons expert Blain (Jesse Ventura) and CIA agent George (Carl Weathers), land in Central America, something is gravely wrong. After finding a string of dead bodies, the crew discovers they are being hunted by a brutal creature with superhuman strength and the ability to disappear into its surroundings."""


def test_devils_lair():
    assert strike.DEVILS_LAIR.name == strike.get_short_name(DEVILS_LAIR)


def test_will_of_crota():
    assert strike.WILL_OF_CROTA.name == strike.get_short_name(WILL_OF_CROTA)


def test_summoning_pits():
    assert strike.SUMMONING_PITS.name == strike.get_short_name(SUMMONING_PITS)


def test_nexus():
    assert strike.NEXUS.name == strike.get_short_name(NEXUS)


def test_winters_run():
    assert strike.WINTERS_RUN.name == strike.get_short_name(WINTERS_RUN)


def test_cerberus_vae_iii():
    assert strike.CERBERUS_VAE_III.name == strike.get_short_name(CERBERUS_VAE_III)


def test_dust_palace():
    assert strike.DUST_PALACE.name == strike.get_short_name(DUST_PALACE)


def test_undying_mind():
    assert strike.UNDYING_MIND.name == strike.get_short_name(UNDYING_MIND)


def test_predator():
    with pytest.raises(strike.UnknownStrikeError):
        strike.get_short_name(PREDATOR)
